<?php
#####rrbranchesqueue_ajx.php, installed by ###Admin Branches Queue, by Robin Richmond:
# This AJAX program can clear labels or add labels for a set of branches.
# It is invoked by the new branch processing queue.
# It loops through the branches and invokes ajx_labels.php as a web page, using the PHP
# function file_get_contents.  As a result, the branches are done in sequence, not in
# parallel, and one program will not accumulate the execution time require to process
# each branch.
#
# Input parameters, many of which are just passed on to ajx_labels.
#  $branchlist - the list of branches, formatted as a comma-delimited string with single
#                quotes around each branch ID.
#  $branch - In the absence of a list, we may be given just one branch name.
#  $branchaction -  'add' or 'clear'
#  $set - Applies only to 'clear'. 'all'=>Don't follow the rule; Look at all People & Families records
#  $overwrite - Needed for add operations to indicate whether or how to overwrite existing labels
#  $summaryonly - When 1, keeps the list of names in the branch from being displayed.
#                 (For now, $summaryonly is always set to 1 in the call to ajx_labels below)
#  $allowNonAdminLogin: - Flag indicating whether the login process (vs the admin app)
#                           called this program.
$tngModNames['Admin Branches Queue']['rrbranchesqueue_ajx.php'] = '5'; #Register this mod
#################################################################

include("begin.php");
include("adminlib.php");
$textpart = "branches";
//include("getlang.php");
include("$mylanguage/admintext.php");

#Read custom language strings from a mod subfolder rather than from regular cust_text.php files.
$modfolder = $admtext['rrbq-modfolder'];
include("$modspath/$modfolder/languages/English_custtext.php");
if ($mylanguage != "languages/English" && file_exists("$modspath/$modfolder/{$mylanguage}_custtext.php"))
	include("$modspath/$modfolder/{$mylanguage}_custtext.php");
#######################################################################

if (empty($branchaction)) {
	echo "<h3>Nothing to do</h3>";
	exit;
}
if (!isset($allowNonAdminLogin)) $allowNonAdminLogin=0;
if (!isset($summaryonly)) $summaryonly=0;
if (!isset($branchlist)) $branchlist='';
if (!isset($branch)) $branch='';
if ($branchaction != 'clear' || empty($set)) $set='';
if (!$branchlist && !$branch) {
	echo "<h3>Nothing to do</h3>";
	exit;
}
if (!$branchlist) $branchlist="'$branch'"; #Form a list from the single branch.

if ( $allowNonAdminLogin ) $admin_login = 0;
else $admin_login = 1;
include("checklogin.php");


@set_time_limit(0);
session_write_close(); # Disable session writes so that other sessions can start.

header("Content-type:text/html; charset=" . $session_charset);

#We must mark branches that need rebuilding to show that they are queued.
$query = "UPDATE $branches_table
	SET username='$currentuser', actionrequested='$branchaction', datequeued=NOW(), datestarted=NULL
	WHERE branch IN ($branchlist)";
tng_query($query);
#We can also now remove the quotes from the list, since we don't need to use it in another SQL IN condition.
$branchlist = str_replace("'", '', $branchlist);
#And now make it an array so we can loop through the branches more easily.
$brancharray = explode(',', $branchlist);

#Heading for the list of branch completion statuses
echo "{$admtext['rrbq-processed']} ($branchaction):
	<ol style='margin-top:0;margin-bottom:0;'>\n";

#Now step through the list of branches. Read one branch at a time to assure that
#we know its status as we're about to process it.
for ($b=0; $b<count($brancharray); $b++) {
	$branch = $brancharray[$b];
	$query = "SELECT * FROM $branches_table WHERE branch ='$branch'";
	$bresult = tng_query($query);
	if (!$bresult) {
		echo "<li>Branch $branch is no longer defined!?!</li>\n";
		continue;
	}
	$brow = tng_fetch_assoc($bresult);
	echo "<li><b>$branch &ndash; {$brow['description']} [[$branchaction]]</b>: ";

	if (!$brow['personID'] || (!$brow['dgens'] && !$brow['agens']) ) {
		echo "No parameters to process; no records added";
	}
	elseif (is_null($brow['datequeued']) ) {
		echo " has been removed from the queue.";
	}
	elseif (!is_null($brow['datestarted']) ) {
		echo " is already being processed.";
	}
	else { #It looks like we're ok to go...
		if ($branchaction == 'clearadd') {
			echo "Clear: ",processABranch('clear', 'all', $branch, $brow);
			$query = "UPDATE $branches_table SET datequeued='NOW'
				WHERE branch = '$branch'";
			tng_query($query);
			echo "<br>Add: ", processABranch('add','',$branch, $brow);
		} else {
			echo processABranch ($branchaction, $set, $branch, $brow);
		}
		echo "\n";
	} #end if we can and need to process this branch
	echo "</li>\n";
} #end of loop through the branch list
echo "</ol>\n";
tng_free_result( $bresult );

function processABranch ($branchaction, $set, $branch, $brow) {
	global $tngdomain;
	#I saw a personID with a trailing space once, so I'm going to trim them all.
	$personID = trim($brow['personID']);
	#Always set overwrite=2, which does not overwrite the Person/Family branch field, but rather adds the new branch to it.
	#It's meaningless for clear operations, but it doesn't hurt.
	#And always set allowNonAdminLogins=1.  It's essential for queued branch operations.
	$myurl = "$tngdomain/ajx_labels.php?branchaction=$branchaction&set=$set&summaryonly=1&overwrite=2&allowNonAdminLogin=1&branch=$branch&tree={$brow['gedcom']}&personID=$personID&agens={$brow['agens']}&dgens={$brow['dgens']}&dagens={$brow['dagens']}&dospouses={$brow['inclspouses']}";
	$webpage = file_get_contents($myurl);
	#Return the output of ajx_labels.php, which should be a count of records processed (and, though we hope not, possibly a list of names).  Or it could be an error mesage.
	return $webpage;
} #End function processABranch

?>